@@ -1,5 +1,5 @@
 #!/usr/bin/python
-# $Id: wlink.py 1395 2016-01-19 15:43:12Z mwall $
+# $Id: wlink.py 1400 2016-01-21 19:15:04Z mwall $
 # Copyright 2014 Matthew Wall
 """weewx driver for WeatherLink web sites
    download data from a weatherlink site for use in weewx
@@ -50,7 +50,7 @@ import weewx
 import weewx.drivers
 
 DRIVER_NAME = "WeatherLink"
-DRIVER_VERSION = "0.7"
+DRIVER_VERSION = "0.9"
 
 if weewx.__version__ < "3":
     raise weewx.UnsupportedFeature("weewx 3 is required, found %s" %
@@ -94,7 +94,8 @@ class WeatherLink(weewx.drivers.AbstractDevice):
         loginf("version is %s" % DRIVER_VERSION)
         self.username = stn_dict['username']
         self.password = stn_dict['password']
-        self.format = float(stn_dict.get('format', 'B'))
+        self.format = stn_dict.get('format', 'B')
+        loginf("expecting HTML format '%s'" % self.format)
         self.poll_interval = float(stn_dict.get('poll_interval', 60))
         self.max_tries = int(stn_dict.get('max_tries', 5))
         self.retry_wait = int(stn_dict.get('retry_wait', 30))
@@ -144,7 +145,7 @@ class WeatherLink(weewx.drivers.AbstractDevice):
                ((len(data) / 52), since_ts))
         sidx = 0
         while sidx < len(data):
-            record = self.unpack_archive_packet(data[sidx:sidx+52])
+            record = self.unpack_archive_packet(data[sidx:sidx + 52])
             if record['dateTime']:
                 yield record
             sidx += 52
@@ -158,7 +159,7 @@ class WeatherLink(weewx.drivers.AbstractDevice):
             except (urllib2.URLError, socket.error,
                     httplib.BadStatusLine, httplib.IncompleteRead), e:
                 logerr('download failed attempt %d of %d: %s' %
-                       (count+1, self.max_tries, e))
+                       (count + 1, self.max_tries, e))
                 time.sleep(self.retry_wait)
         else:
             logerr('download failed after %d tries' % self.max_tries)
@@ -169,6 +170,8 @@ class WeatherLink(weewx.drivers.AbstractDevice):
         packet = {}
         if text is not None:
             parser = WLParserB()
+            if self.format == 'A':
+                parser = WLParserA()
             parser.feed(text)
             packet = parser.get_data()
         return packet
@@ -200,9 +203,9 @@ class WeatherLink(weewx.drivers.AbstractDevice):
         data_tuple = archive_format.unpack(raw_record_string)
         raw_record = dict(zip(data_types, data_tuple))
         packet = {
-            'dateTime' : _archive_datetime(raw_record['date_stamp'],
-                                           raw_record['time_stamp']),
-            'usUnits' : weewx.US,
+            'dateTime': _archive_datetime(raw_record['date_stamp'],
+                                          raw_record['time_stamp']),
+            'usUnits': weewx.US,
             }
         for t in raw_record:
             func = _archive_map.get(t)
@@ -221,12 +224,12 @@ class WeatherLink(weewx.drivers.AbstractDevice):
 def _epoch_to_timestamp(epoch):
     """convert unix epoch to davis timestamp"""
     tt = time.localtime(epoch)
-    ds = tt[2] + (tt[1]<<5) + ((tt[0]-2000)<<9)
-    ts = tt[3]*100 + tt[4]
-    x = (ds<<16) | ts
+    ds = tt[2] + (tt[1] << 5) + ((tt[0] - 2000) << 9)
+    ts = tt[3] * 100 + tt[4]
+    x = (ds << 16) | ts
     return x
 
-def _archive_datetime(datestamp, timestamp) :
+def _archive_datetime(datestamp, timestamp):
     """Returns the epoch time of the archive packet."""
     try:
         time_tuple = ((0xfe00 & datestamp) >> 9,    # year
@@ -248,10 +251,10 @@ def _stime(v):
     # Return seconds since midnight
     return 3600*h + 60*m
 
-def _big_val(v) :
+def _big_val(v):
     return float(v) if v != 0x7fff else None
 
-def _big_val10(v) :
+def _big_val10(v):
     return float(v)/10.0 if v != 0x7fff else None
 
 def _big_val100(v):
@@ -260,25 +263,25 @@ def _big_val100(v):
 def _val100(v):
     return float(v)/100.0
 
-def _val1000(v) :
+def _val1000(v):
     return float(v)/1000.0
 
 def _val1000Zero(v):
     return float(v)/1000.0 if v != 0 else None
 
-def _little_val(v) :
+def _little_val(v):
     return float(v) if v != 0x00ff else None
 
-def _little_val10(v) :
+def _little_val10(v):
     return float(v)/10.0 if v != 0x00ff else None
 
-def _little_temp(v) :
+def _little_temp(v):
     return float(v-90) if v != 0x00ff else None
 
 def _null(v):
     return v
 
-def _null_float(v) :
+def _null_float(v):
     return float(v)
 
 def _null_int(v):
@@ -376,10 +379,71 @@ rec_types_B, fmt_B = zip(*rec_format_B)
 rec_fmt_A = struct.Struct('<' + ''.join(fmt_A))
 rec_fmt_B = struct.Struct('<' + ''.join(fmt_B))
 
-compass_points = {'N':0,'NNE':22.5,'NE':45,'ENE':67.5,'E':90,'ESE':112.5,'SE':135,'SSE':157.5,'S':180,'SSW':202.5,'SW':225,'WSW':247.5,'W':270,'WNW':292.5,'NW':315,'NNW':337.5}
+compass_points = {'N': 0, 'NNE': 22.5, 'NE': 45, 'ENE': 67.5, 'E': 90,
+                  'ESE': 112.5, 'SE': 135, 'SSE': 157.5, 'S': 180,
+                  'SSW': 202.5, 'SW': 225, 'WSW': 247.5, 'W': 270,
+                  'WNW': 292.5, 'NW': 315, 'NNW': 337.5}
 
 # crude parser to get data from the weatherlink web page.
-# works with weatherlink html as of 2016 (pages changed sometime late 2015)
+# works with html up to at least late 2015
+class WLParserA(HTMLParser):
+    def __init__(self):
+        HTMLParser.__init__(self)
+        self.packet = {}
+        self.hierarchy = []
+        self.last_value = None
+        self.last_key = None
+
+    def handle_starttag(self, tag, attrs):
+        if tag not in ['meta', 'link', 'br']:
+            self.hierarchy.append(tag)
+
+    def handle_endtag(self, tag):
+        self.hierarchy.pop()
+
+    def handle_data(self, data):
+        data = data.strip()
+        if not 'span' in self.hierarchy:
+            if len(data):
+                self.last_key = data
+            return
+        if data in compass_points:
+            self.packet['windDir'] = compass_points[data]
+        elif self.last_value is not None:
+            if data == self.last_value:
+                try:
+                    v = float(self.last_value)
+                    self.packet['outTemperature'] = v
+                except ValueError:
+                    pass
+            elif data == 'km/h' or data == 'Mph':
+                self.packet['windSpeed'] = float(self.last_value)
+            elif data == '%':
+                self.packet['outHumidity'] = float(self.last_value)
+            self.last_value = None
+        elif re.search('\d+mm', data):
+            m = re.search('(\d+)mm', data)
+            self.packet['rain'] = float(m.group(1)) / 10 # weewx wants cm
+        elif re.search('[\d.]+mb', data):
+            m = re.search('([\d.]+)mb', data)
+            self.packet['barometer'] = float(m.group(1))
+        elif re.search('[\d.]+hPa', data):
+            m = re.search('([\d.]+)hPa', data)
+            self.packet['barometer'] = float(m.group(1))
+        elif re.search('[\d.]+"', data):
+            m = re.search('([\d.]+)"', data)
+            if self.last_key == 'Rain':
+                self.packet['rain'] = float(m.group(1))
+            elif self.last_key == 'Barometer':
+                self.packet['barometer'] = float(m.group(1))
+            self.last_key = None
+        else:
+            self.last_value = data
+
+    def get_data(self):
+        return self.packet
+
+# works with weatherlink html as of 2016
 # pick off temperature, wind, humidity, rain, barometer
 class WLParserB(HTMLParser):
     def __init__(self):
@@ -389,7 +453,7 @@ class WLParserB(HTMLParser):
         self.last_key = 'Temperature'
 
     def handle_starttag(self, tag, attrs):
-        if tag not in ['meta','link','br']:
+        if tag not in ['meta', 'link', 'br']:
             self.hierarchy.append(tag)
 
     def handle_endtag(self, tag):
@@ -419,9 +483,9 @@ class WLParserB(HTMLParser):
                     self.packet['windDir'] = compass_points[wdir]
                 logdev('windDir: %s' % self.packet.get('windDir'))
                 # do not reset last_key since we still need a wind speed
-            elif re.search('\d+', data):
-                m = re.search('(\d+)', data)
-                self.packet['windSpeed'] = m.group(1)
+            elif re.search('[\d.]+', data):
+                m = re.search('([\d.]+)', data)
+                self.packet['windSpeed'] = float(m.group(1))
                 logdev('windSpeed: %s' % self.packet.get('windSpeed')) 
                 self.last_key = None
         if self.last_key == 'Temperature':
@@ -430,15 +494,15 @@ class WLParserB(HTMLParser):
                 self.packet['outTemperature'] = v
                 logdev('outTemperature: %s' % self.packet['outTemperature'])
                 self.last_key = None
-            except:
+            except ValueError:
                 pass
         if self.last_key == 'Humidity':
             self.packet['outHumidity'] = float(data)
             logdev('outHumidity: %s' % self.packet['outHumidity'])
             self.last_key = None
         if self.last_key == 'Rain':
-            if re.search('\d+mm', data):
-                m = re.search('(\d+)mm', data)
+            if re.search('[\d.]+mm', data):
+                m = re.search('([\d.]+)mm', data)
                 self.packet['rain'] = float(m.group(1)) / 10 # weewx wants cm
                 logdev('rain: %s' % self.packet['rain'])
             elif re.search('[\d.]+"', data):
@@ -480,6 +544,8 @@ if __name__ == "__main__":
                       help='test the parser')
     parser.add_option('--filename', dest='filename', default='testfile.xml',
                       help='name of file for test-parser')
+    parser.add_option('--format', dest='format', default='B',
+                      help='html format, can be A or B')
     (options, args) = parser.parse_args()
     syslog.setlogmask(syslog.LOG_UPTO(syslog.LOG_DEBUG))
     if options.test_parser:
@@ -488,6 +554,8 @@ if __name__ == "__main__":
             for line in f:
                 data.append(line)
         parser = WLParserB()
+        if options.format == 'A':
+            parser = WLParserA()
         parser.feed(''.join(data))
         print parser.get_data()
     elif options.test_driver:
